@@ -132,7 +132,7 @@ class BaseSchemaLoader:
             except Resolver404:
                 continue
             else:
-                for key, value in resolved_route.kwargs.items():
+                for key, value in reversed(list(resolved_route.kwargs.items())):
                     index = path.rfind(str(value))
                     path = f"{path[:index]}{{{key}}}{path[index + len(str(value)):]}"
                 if "{pk}" in path and api_settings.SCHEMA_COERCE_PATH_PK:
@@ -231,6 +231,6 @@ class StaticSchemaLoader(BaseSchemaLoader):
         :return: Schema contents as a dict
         :raises: ImproperlyConfigured
         """
-        with open(self.path) as file:
+        with open(self.path, encoding="utf-8") as file:
             content = file.read()
             return json.loads(content) if ".json" in self.path else yaml.load(content, Loader=yaml.FullLoader)
